/*
  Module Electro
  part of Arduino Mega Server project
    ESP_32
*/

#ifdef FEATURE_ELECTRO
const int MAX_UI_SENSORS = 2; // 2 - 12

#include "EmonLib.h"             // Include Emon Library
EnergyMonitor emon1;             // Create an instance


void initElectro() {

  
  emon1.voltage(36, 234.26, 1.7);  // Voltage: input pin, calibration, phase_shift
  emon1.current(39, 111.1);       // Current: input pin, calibration.

  moduleElectro = ENABLE;

  buttonElectro = 1;//========================
  
  started(F("Electro"), true);
}

void workElectro() {

  emon1.calcVI(20,2000);         // Calculate all. No.of half wavelengths (crossings), time-out
  emon1.serialprint();           // Print out all variables (realpower, apparent power, Vrms, Irms, power factor)
  
  float realPower       = emon1.realPower;        //extract Real Power into variable
  float apparentPower   = emon1.apparentPower;    //extract Apparent Power into variable
  float powerFActor     = emon1.powerFactor;      //extract Power Factor into Variable
  float supplyVoltage   = emon1.Vrms;             //extract Vrms into Variable
  float Irms            = emon1.Irms;             //extract Irms into Variable
}


// U & I signal form
#define MAX_FORM_POINTS 90
int pointsU[MAX_FORM_POINTS];
int pointsI[MAX_FORM_POINTS];

// Power supply frequency
#define MAX_FREQS 206
long periodInMicros = 0;
int cyclosPerPeriod = 0;

/* ----------------------------------------
  Function synchroSignal()
    Synchronization of signal power supply
    "down" - "up" (start: middle ~ 50%)
------------------------------------------- */

const int uPin          = 36;  //  IO32

void synchroSignal(byte uPin, int limitCyclos) {

 // void synchroSignal(byte 35, int limitCyclos) {
  int val;
  int i = 0;
  int j = 0;
  /* 
    Limits of signal:
    top -    826
    middle - 560
    bottom - 294
  */
  int limitBottomArea = 310; // 800
  int limitMiddleArea = 540; // 560

  do { // check top area
   // val = analogRead(uPin);

    val = analogRead(36);
    //delayMicroseconds(delayDue);
    i++; if (i > limitCyclos) {break;}
  } while (val > limitBottomArea);
  
  do { // check top of middle area
   // val = analogRead(uPin);
     val = analogRead(36);
    //delayMicroseconds(delayDue);
    j++; if (j > limitCyclos) {break;}    
  } while (val < limitMiddleArea);
} // synchroSignal( )

/* ----------------------------------------
  Function getFormArrays
    Get arrays of points U & I for graphics
------------------------------------------- */

void getFormArrays(byte pinU, byte pinI) {

//  void getFormArrays(35, 39) {
  synchroSignal(pinU, MAX_FORM_POINTS);

  synchroSignal(36, MAX_FORM_POINTS);
  for (int i = 0; i < MAX_FORM_POINTS; i++) {
   // pointsU[i] = analogRead(pinU);
    pointsU[i] = analogRead(36);
    //pointsI[i] = analogRead(pinI);
    pointsI[i] = analogRead(39);
   // delayMicroseconds(delayDue);
  }
}

/* ----------------------------------------
  Function getFreqPeriod
    Get periodInMicros and cyclosPerPeriod
    of U signal
------------------------------------------- */

void getFreqPeriod(byte pinU) {

//  void getFreqPeriod(byte 35) {
  unsigned long startTime;
  int startVal;
  int tempVal;

  periodInMicros = 0;
  cyclosPerPeriod = 0;
  synchroSignal(pinU, MAX_FREQS);

  //synchroSignal(35, MAX_FREQS);
  
  startTime = micros();
  //startVal = analogRead(pinU);

   startVal = analogRead(36);
  for (int i = 0; i < MAX_FREQS; i++) {
   // tempVal = analogRead(pinU);
    tempVal = analogRead(36);
//    delayMicroseconds(delayDue / 2);
    if ((i > 146 && i < 206) && (tempVal < startVal + 10 && tempVal > startVal - 10)) {
      periodInMicros = micros() - startTime;
      cyclosPerPeriod = i;
      break;
    }
  }
} // getFrecPeriod( )

/* -----------------
  Function freqWorks
-------------------- */

void freqWorks() {
  getFormArrays(0, 1);
  getFreqPeriod(0);
}


#endif

