/*
  Modul nRF24
  part of Arduino Mega Server project
*/

#ifdef FEATURE_NRF24

//#include "esp32-hal-spi.h"

#include <SPI.h>
#include "RF24.h"

// nRF24L01 modul configuration
//RF24 radio(8, 9); // CE, CSN pins
//RF24 radio(0, 2); // CE, CSN pins
RF24 radio(14, 12); // CE, CSN pins
byte addresses[][6] = {"1Node", "2Node"};

// packet buffers
#define PACKET_MAX_BYTES 32
byte buffTx[PACKET_MAX_BYTES];
byte buffRx[PACKET_MAX_BYTES];

// packet structure
#define POS_TYPE      0
#define POS_COMMAND   1
#define POS_ADDR      2
#define POS_DEST      3
#define POS_REQUIRED  4
#define POS_NUMBER    5
#define POS_PASSWORD  9
#define POS_DATA      17

// packet type
#define TYPE_EMPTY   0
#define TYPE_COMMAND 1
#define TYPE_REQUEST 2
#define TYPE_DATA    3
#define TYPE_ALARM   4
#define TYPE_ANSWER  5
#define TYPE_ECHO    6

// packet variables
byte   rxType;
byte   rxCommand;
byte   rxAddr;
byte   rxDestination;
byte   rxRequired;
byte   rxNumber;
String rxPassword;
String rxData;

// required answer
#define NOT_REQUIRED 0
#define REQUIRED     1

// logic
#define NOT_DEF 0
#define OUT     0
#define IN      1


  // commands
  #define GET_NAME      1
  #define DTA_NAME      2
  #define SET_NAME      3
  #define GET_ADDR      4
  #define DTA_ADDR      5
  #define SET_ADDR      6
  #define GET_FREE_MEM  7
  #define DTA_FREE_MEM  8
  #define GET_BATTERY   9
  #define DTA_BATTERY   10
  #define GET_SELF_TEMP 11
  #define DTA_SELF_TEMP 12
  #define GET_TEMP_1    13
  #define DTA_TEMP_1    14

  #define GET_TEMP_2    17     //      или 2 температуры или температура и вода
  #define DTA_TEMP_2    18     //
  
  #define GET_TEMP_DHT  23     //
  #define DTA_TEMP_DHT  24     //

  #define GET_HUM_DHT   25     //
  #define DTA_HUM_DHT   26     //
  
  #define GET_VOL_1     19     //      запрос объема
  #define DTA_VOL_1     20     //      объем воды от YFS201

  
  #define GET_VOL_2     21// добавляем переменные для счетчика 2 литров-запрос
  #define DTA_VOL_2      22// Тоже данные
 
  #define GET_HEALTH    15
  #define DTA_HEALTH    16
  #define GET_ID        100
  #define DTA_ID        101
  #define SET_ID        102
  #define GEN_ID        103
  #define BAK_ECHO      200

  // sender structure
  typedef struct Sender {
    boolean active      = false;
    boolean required    = true;
    boolean success     = false;
    byte type           = 0;
    byte command        = 0;
    byte number         = 0;
    byte tryouts        = 0;
    int errors          = 0;
    String result       = "";
    unsigned long timer = millis();
  } sender;

  sender sendEcho,
         sendGetName, sendGetId, sendGetAddress,
         sendGetFreeMem, sendGetBattery, sendGetSelfTemp,
         sendGetHealth, sendGetVol1,sendGetVol2,
         sendGetTemp1,
         sendGetTemp2,
         sendGetDHTtemp, sendGetDHThum;
         

 // net stat
 int sendPackets        = 0;
 int sendRetrys         = 0;
 int sendErrors         = 0;
 int receivePackets     = 0;
 int successPackets     = 0;
 int sendPacketsStat    = 0;
 int sendRetrysStat     = 0;
 int sendErrorsStat     = 0;
 int receivePacketsStat = 0;
 int successPacketsStat = 0;
    


// nRF24 controller
  String nrf1Name;
  String nrf1Id;
  byte   nrf1Addr;
  byte   nrf1Battery;
  byte   nrf1Health;
  int    nrf1FreeMem;
  float  nrf1SelfTemp;
  float  nrf1Temp1;
  float  nrf1Temp2;
// float  nrf1Temp3;
  
  float  nrf1DHTtemp;
  float  nrf1DHThum;
    
  unsigned int nrf1Vol1;  
 
  unsigned int nrf1Vol2; 


void InitNrf24() {
  //initStart("nRF24", true);

//  initStart("nRF24");
  timeStamp();
  Serial.println(F(" Base Receiving"));
  radio.begin();
  radio.setPALevel(RF24_PA_LOW); // default RF24_PA_MAX
  
  radio.openWritingPipe(addresses[0]);
  radio.openReadingPipe(1, addresses[1]);
    
  radio.startListening();

  //serialDynamic = ACTIVE;
  moduleNrf24 = ENABLE;
 // initDone(true);

 started(F("nRF24"), true);
}

#ifdef SERIAL_PRINT
  void printBuff(byte buff[]) {
    for (byte i = 0; i < POS_PASSWORD; i++) {
      if      (buff[i] < 10)                 {Serial.print(F("   "));}
      else if (buff[i] > 9 && buff[i] < 100) {Serial.print(F("  "));}
      else if (buff[i] > 99)                 {Serial.print(F(" "));}
      Serial.print(buff[i]);
    }
    Serial.print(F(" "));
    for (byte i = POS_PASSWORD; i < POS_DATA; i++) {
      if (buff[i] == 0) {break;}
      Serial.print(char(buff[i]));
    }
    Serial.print(F(" "));
    for (byte i = POS_DATA; i < PACKET_MAX_BYTES; i++) {
      if (buff[i] == 0) {break;}
      Serial.print(char(buff[i]));
    }
  } // printBuff( )
#endif // SERIAL_PRINT

// set packet parameters

void setNotRequired(struct Sender *p) {
  p->required = NOT_REQUIRED;
}

void setPacketType(struct Sender *p, byte type) {
  p->type = type;
}

void setNextNumber(struct Sender *p) {
  p->number++;
  if (p->number > 255) {
    p->number = 0;
  }
}

boolean success(struct Sender *p) {
  return p->success;
}

// Make echo

void makeEcho() {
  for (byte i = 0; i < PACKET_MAX_BYTES; i++) {
    buffTx[i] = buffRx[i];
  }
  sendEcho.required = false;
  buffTx[POS_TYPE] = TYPE_ECHO;
}

// parse buffer

void parseBuffRx() {
  rxType        = buffRx[POS_TYPE];
  rxCommand     = buffRx[POS_COMMAND];
  rxAddr        = buffRx[POS_ADDR];
  rxDestination = buffRx[POS_DEST];
  rxRequired    = buffRx[POS_REQUIRED];
  rxNumber      = buffRx[POS_NUMBER];

  rxPassword = "";
  for (byte i = POS_PASSWORD; i < POS_DATA; i++) {
    if (buffRx[i] == 0) {break;}
    rxPassword += char(buffRx[i]);
  }
  rxData = "";
  for (byte i = POS_DATA; i < PACKET_MAX_BYTES; i++) {
    if (buffRx[i] == 0) {break;}
    rxData += char(buffRx[i]);
  }
}

#ifdef SERIAL_PRINT
  void printPacketType(byte type, byte dir) {
    String arrow;
    if (dir == IN) {arrow = F("<-");}
              else {arrow = F("->");}
  
    switch (type) {
      case TYPE_EMPTY:   Serial.print(F("Empty:   ")); Serial.print(arrow); break;
      case TYPE_COMMAND: Serial.print(F("Command: ")); Serial.print(arrow); break;
      case TYPE_REQUEST: Serial.print(F("Request: ")); Serial.print(arrow); break;
      case TYPE_DATA:    Serial.print(F("Data:    ")); Serial.print(arrow); break;
      case TYPE_ALARM:   Serial.print(F("Alarm:   ")); Serial.print(arrow); break;
      case TYPE_ANSWER:  Serial.print(F("Answer:  ")); Serial.print(arrow); break;
      case TYPE_ECHO:    Serial.print(F("Echo:    ")); Serial.print(arrow); break;
      default: Serial.print(F("Unknown: "));
    }
  }
#endif

void saveResult() {
  switch (rxCommand) {
    case DTA_NAME:      sendGetName.result     = rxData; break;
    case DTA_ID:        sendGetId.result       = rxData; break;
    case DTA_ADDR:      sendGetAddress.result  = rxData; break;
    case DTA_FREE_MEM:  sendGetFreeMem.result  = rxData; break;
    case DTA_BATTERY:   sendGetBattery.result  = rxData; break;
    case DTA_SELF_TEMP: sendGetSelfTemp.result = rxData; break;
    case DTA_HEALTH:    sendGetHealth.result   = rxData; break;
    case DTA_TEMP_1:    sendGetTemp1.result    = rxData; break;

    case DTA_TEMP_2:    sendGetTemp2.result    = rxData; break;
  //case DTA_TEMP_3:    sendGetTemp3.result    = rxData; break;
    
    case DTA_VOL_1:    sendGetVol1.result    = rxData; break;


    case DTA_VOL_2:    sendGetVol2.result    = rxData; break;

    case DTA_TEMP_DHT:    sendGetDHTtemp.result    = rxData; break;


    case DTA_HUM_DHT:    sendGetDHThum.result    = rxData; break;
  }
}

void WorksNrf24() {
  if (radio.available()) {
    while (radio.available()) {
      radio.read(&buffRx, PACKET_MAX_BYTES);
    }
    
    receivePackets++;
    parseBuffRx();
    saveResult();

    #ifdef SERIAL_PRINT
      printPacketType(rxType, IN);
      printBuff(buffRx);
      Serial.println();
    #endif

    switch (rxType) {
      case TYPE_COMMAND: parseCommands(); break;
      case TYPE_DATA:    parseData();     break;
      case TYPE_ECHO:    parseEcho();     break;
      case TYPE_ANSWER:  parseAnswers();  break;
      case TYPE_REQUEST: parseRequests(); break;
      case TYPE_ALARM:   parseAlarms();   break;
      default: ;
    }
  } // if (radio.available())
  
  retrysControl();
} // nrf24Works()

// carousel
byte currPony = 0;

void carousel() {
  #define MAX_PONY 13
  currPony++;
  if (currPony > MAX_PONY) {currPony = 1;}
  switch (currPony) {
    case 1: if (nrf1Name     == "") {sendPacket(GET_NAME);}      break;
    case 2: if (nrf1Id       == "") {sendPacket(GET_ID);}        break;
    case 3: if (nrf1Addr     == 0)  {sendPacket(GET_ADDR);}      break;
    case 4: if (nrf1FreeMem  == 0)  {sendPacket(GET_FREE_MEM);}  break;
    case 5: if (nrf1Battery  == 0)  {sendPacket(GET_BATTERY);}   break;
    case 6: if (nrf1SelfTemp == 0)  {sendPacket(GET_SELF_TEMP);} break;
    case 7: if (nrf1Health   == 0)  {sendPacket(GET_HEALTH);}    break;
    case 8: if (nrf1Temp1    == 0)  {sendPacket(GET_TEMP_1);}    break;

    case 9: if (nrf1Temp2    == 0)  {sendPacket(GET_TEMP_2);}    break;
  //case 10: if (nrf1Temp3   == 0)  {sendPacket(GET_TEMP_3);}    break;
    
    case 10: if (nrf1Vol1    == 0)  {sendPacket(GET_VOL_1);}    break;

    case 11: if (nrf1Vol2    == 0)  {sendPacket(GET_VOL_2);}    break;

    case 12: if (nrf1DHTtemp  == 0)  {sendPacket(GET_TEMP_DHT);}    break;

    case 13: if (nrf1DHThum   == 0)  {sendPacket(GET_HUM_DHT);}    break;
  }
}

#endif // NRF24_FEATURE
