/*
  Modul nRF24 Check
  part of Arduino Mega Server project
*/
#ifdef FEATURE_NRF24
//#ifdef NRF24_FEATURE

void resetSender(struct Sender *p) {
  p->active   = false;
  p->required = true;
  p->success  = true;
  p->tryouts  = 0;
  p->type     = NOT_DEF;
  setNextNumber(p);
}

// Echo

boolean matchEcho(struct Sender *p) {
  if (rxNumber != p->number) {
    return false;
  }
  return true;
}

void checkEcho(struct Sender *p) {
  if (matchEcho(p)) {
    resetSender(p);
    successPackets++;
    #ifdef SERIAL_PRINT
      Serial.print(F("ECHO: "));
      Serial.print(p->command);
      Serial.println(F(" DONE"));
    #endif
  }
}

void parseEcho() {
  switch (rxCommand) {
    case GET_NAME:      checkEcho(&sendGetName);     break;
    case GET_ID:        checkEcho(&sendGetId);       break;
    case GET_ADDR:      checkEcho(&sendGetAddress);  break;
    case GET_FREE_MEM:  checkEcho(&sendGetFreeMem);  break;
    case GET_BATTERY:   checkEcho(&sendGetBattery);  break;
    case GET_SELF_TEMP: checkEcho(&sendGetSelfTemp); break;
    case GET_HEALTH:    checkEcho(&sendGetHealth);   break;
    case GET_TEMP_1:    checkEcho(&sendGetTemp1);    break;
    case GET_TEMP_2:    checkEcho(&sendGetTemp2);    break;        
  //case GET_TEMP_3:    checkEcho(&sendGetTemp3);    break;   
    case GET_VOL_1:    checkEcho(&sendGetVol1);    break;   
    case GET_VOL_2:    checkEcho(&sendGetVol2);    break; 

    case GET_TEMP_DHT:    checkEcho(&sendGetDHTtemp);    break;   
   
    case GET_HUM_DHT:    checkEcho(&sendGetDHThum);    break;  
        
  }
}

// Answers

void checkAnswer(struct Sender *p) {
  resetSender(p);
  successPackets++;
  #ifdef SERIAL_PRINT
    Serial.print(F("ANSWER: "));
    Serial.print(p->command);
    Serial.println(F(" DONE"));
  #endif  
}

void parseAnswers() {
  switch (rxCommand) {
    case DTA_NAME:
      checkAnswer(&sendGetName);
      nrf1Name = sendGetName.result;
      break;
    case DTA_ID:
      checkAnswer(&sendGetId);
      nrf1Id = sendGetId.result;
      break;
    case DTA_ADDR:
      checkAnswer(&sendGetAddress);
      nrf1Addr = sendGetAddress.result.toInt();
      break;
    case DTA_FREE_MEM:
      checkAnswer(&sendGetFreeMem);
      nrf1FreeMem = sendGetFreeMem.result.toInt();
    case DTA_BATTERY:
      checkAnswer(&sendGetBattery);
      nrf1Battery = sendGetBattery.result.toInt();
      break;
    case DTA_SELF_TEMP:
      checkAnswer(&sendGetSelfTemp);
      nrf1SelfTemp = sendGetSelfTemp.result.toFloat();
      break;
    case DTA_HEALTH:
      checkAnswer(&sendGetHealth);
      nrf1Health = sendGetHealth.result.toInt();
      break;
    case DTA_TEMP_1:
      checkAnswer(&sendGetTemp1);
      nrf1Temp1 = sendGetTemp1.result.toFloat();
      break;


          case DTA_TEMP_2:
      checkAnswer(&sendGetTemp2);
      nrf1Temp2 = sendGetTemp2.result.toFloat();
      break;
       
       /*
    case DTA_TEMP_3:
      checkAnswer(&sendGetTemp3);
      nrf1Temp3 = sendGetTemp3.result.toFloat();
      break;
      */
      
    case DTA_VOL_1:
      checkAnswer(&sendGetVol1);
      nrf1Vol1 = sendGetVol1.result.toFloat();
      break;

   case DTA_VOL_2:
      checkAnswer(&sendGetVol2);
      nrf1Vol2 = sendGetVol2.result.toFloat();
      break;
          
    case DTA_TEMP_DHT :
      checkAnswer(&sendGetDHTtemp);
      nrf1DHTtemp = sendGetDHTtemp.result.toFloat();
      break;
      
          
    case DTA_HUM_DHT:
      checkAnswer(&sendGetDHThum);
      nrf1DHThum = sendGetDHThum.result.toFloat();
      break;
  }
}

#endif // NRF24_FEATURE
