/*
  Module Temperature
  part of Arduino Mega Server project
  ESP 32 DS2482 & DS2423

*/

#ifdef FEATURE_TEMP
#include "OneWire.h"
#include "Wire.h"
#include "DS2482.h"
#include "DS18B20_DS2482.h"

#include "DS2423.h"
DS2482 DS2482(0);
DS18B20_DS2482 DS18B20_devices(&DS2482);

//DS2482 DS2482(0);
OneWire oneWire;
//====================================================================
DeviceAddress counter = {29, 187, 48, 10, 0, 0, 0, 163};
ds2423 myCounter(&oneWire, counter);

//ds2423 myCounter(&DS2423, counter);
//========================================================

#define MAX_ID_DIGITS     8
#define FULL_HEALTH       100
#define ZERO_HEALTH       0
#define MAX_TEMP_NAME_LEN 12
#define MAX_TEMP_SENSORS  4
byte tempSensorHealth[MAX_TEMP_SENSORS];
byte tempModuleHealth;

//========================================
#define MAX_COUNT_NAME_LEN 12
#define MAX_COUNT_SENSORS  5
//byte tempSensorHealth[MAX_TEMP_SENSORS];
//byte tempModuleHealth;
byte  _count[MAX_COUNT_SENSORS][MAX_ID_DIGITS] = {
  {29, 187, 48, 10, 0, 0, 0, 163},
  {29, 187, 48, 10, 0, 0, 0, 163},
  {29, 187, 48, 10, 0, 0, 0, 163},
  {29, 187, 48, 10, 0, 0, 0, 163},
  {29, 187, 48, 10, 0, 0, 0, 163}
};
float curr_count[MAX_COUNT_SENSORS][2];
unsigned long _curr_count[MAX_COUNT_SENSORS][2];
char objCount[MAX_COUNT_SENSORS][MAX_TEMP_NAME_LEN] = {"count1", "count2", "count3", "count4", "count5"};
//================================

DeviceAddress tempSensor[MAX_TEMP_SENSORS];

char objSens[MAX_TEMP_SENSORS][MAX_TEMP_NAME_LEN] = {"tempSTR", "tempHOM", "tempWRM", "tempB2"};

byte idTempDat[MAX_TEMP_SENSORS][MAX_ID_DIGITS] = { 
  {40, 26, 27, 42, 7, 0, 0, 151},   // tempSTR
  {40, 67, 255, 28, 7, 0, 0, 171},// tempHOM
  {40, 239, 16, 29, 7, 0, 0, 176}, // tempWRM
  {40, 255, 237, 27, 3, 22, 3, 146}, // tempBOILER2
};

float current_count1, current_count2;

//float current_count2;

float current_temp[MAX_TEMP_SENSORS];
int temp_[MAX_TEMP_SENSORS];

void initTemp() {
  Wire.begin();
  DS2482.reset();
  DS18B20_devices.begin();
  timeStamp();
  DS2482.wireResetSearch();
  timeStamp();
  Serial.print(F("Found "));
  Serial.print(DS18B20_devices.getDeviceCount());
  Serial.println(F(" devices"));

  for (byte i = 0; i < MAX_TEMP_SENSORS; i++) {
    DS18B20_devices.getAddress(tempSensor[i], i);
    tempSensorHealth[i] = FULL_HEALTH;
  }
  tempModuleHealth = FULL_HEALTH;
  DS2482.wireResetSearch();
  showSensorsId();//==========================================================
  moduleTemp = ENABLE;
  started(F("Temp"), true);
}

void showSensorsId() {
  for (byte n = 0; n < MAX_TEMP_SENSORS; n++) {
    timeStamp();
    Serial.print(F("#"));
    Serial.print(n);
    Serial.print(F(":"));
    for (byte i = 0; i < MAX_ID_DIGITS; i++) {
      Serial.print(" ");
      Serial.print(tempSensor[n][i]);
    }
    Serial.print(" ");
    Serial.println(objSens[n]);
  }
}

void setModuleTempHealth() {
  int sumSensorsHealth = 0;
  for (byte i = 0; i < MAX_TEMP_SENSORS; i++) {
    sumSensorsHealth += tempSensorHealth[i];
  }
  tempModuleHealth = sumSensorsHealth / MAX_TEMP_SENSORS;
}

void workTemp() {
  if (cycle20s) {
    DS18B20_devices.requestTemperatures();
    getTemperature(0); // STR
    getTemperature(1); // HOM (SRV)
    getTemperature(2); // WRM 
    getTemperature(3); //  BOILER2  
    #ifdef TEMP_DEBUG
      for (byte i = 0; i < MAX_TEMP_SENSORS; i++) {
        timeStamp();
        Serial.print(F("Sensor "));
        Serial.print(objSens[i]);
        Serial.print(F(" health: "));
        Serial.println(tempSensorHealth[i]);
      }
      setModuleTempHealth();
      timeStamp();
      Serial.print(F("Modile Temperature Health: "));
      Serial.println(tempModuleHealth);
    #endif // TEMP_DEBUG

  int a =(myCounter.readCounter(1)/430);
  int b =(myCounter.readCounter(2)/430);


  current_count1 = (myCounter.readCounter(1)/430); // жесть - сам придумал!
 //current_count1 = a;

  //сurrent_count2 = (myCounter.readCounter(2)/430);
  current_count2 = b;

  Serial.print("Counter A: ");
  Serial.println(a);
  Serial.print("Counter B: ");
  Serial.println(b);
  Serial.println();
  }
}

void getTemperature(byte n) {
  if (tempSensor[n][1] == idTempDat[n][1] && 
      tempSensor[n][2] == idTempDat[n][2] &&
      tempSensor[n][3] == idTempDat[n][3] && 
      tempSensor[n][4] == idTempDat[n][4]) {
    tempSensorHealth[n] = FULL_HEALTH;
    current_temp[n] = DS18B20_devices.getTempC(tempSensor[n]);

    if (current_temp[n] != -127) {
      if (current_temp[n] != 85) {
        timeStamp();
        Serial.print(objSens[n]);
        Serial.print(": ");
        Serial.println(current_temp[n]);

        temp_[n] = (current_temp[n] - (int)current_temp[n]) * 100;

        if ((current_temp[n] > -1) && (current_temp[n] < 0)) {
          #ifdef FEATURE_MAJOR
            sendRequestMinusM(objSens[n], "-", (int)current_temp[n], abs(temp_[n])); 
          #endif          
        } else {
            #ifdef FEATURE_MAJOR
              sendRequestMinusM(objSens[n], "", (int)current_temp[n], abs(temp_[n])); 
            #endif
          }

      } else { // current_temp[n] = 85
          tempSensorHealth[n] = ZERO_HEALTH;
          timeStamp();
          Serial.print(objSens[n]);
          Serial.print(": ");
          Serial.println(F("error")); 
        }  
    } else { // current_temp[n] = -127
        tempSensorHealth[n] = ZERO_HEALTH;
        timeStamp();
        Serial.print(objSens[n]);
        Serial.print(": ");
        Serial.println(F("removed")); 
      }
  } else { // current Sensor ID != stored Sensor ID
      tempSensorHealth[n] = ZERO_HEALTH;
      timeStamp();
      Serial.print(objSens[n]);
      Serial.print(": ");
      Serial.println(F("ID problem or not present"));
      workCount();
  }
}

void workCount() {
  Serial.println();
  Serial.println("*** workCount ***");
  for (int i = 0; i < MAX_COUNT_SENSORS; i++) {

    timeStamp();//======================================
    
   // Serial.print("("); Serial.print(i); Serial.print(") ");//======================================
   // Serial.print("name: "); Serial.println(objCount[i]);

     //Serial.println(objCount[i]);

     Serial.print(objCount[i]);

     Serial.print(": ");//===============================
    for (int q=0; q < 2; q++) {
      uint8_t data[45];
      DS2482.wireReset();
      DS2482.wireSelect(_count[i]);
      DS2482.wireWriteByte(0xa5);
      if (q == 0) { DS2482.wireWriteByte(0xc0); }
      if (q == 1) { DS2482.wireWriteByte(0xe0); } 
      DS2482.wireWriteByte(0x01);
      for (int j = 3; j < 45; j++) {
        data[j] = DS2482.wireReadByte();
      }
      DS2482.wireReset();
      uint32_t count = (uint32_t)data[38];
      for (int j = 37; j >= 35; j--) {
        count = (count << 8) + (uint32_t)data[j];
        curr_count[i][q] = count;
        _curr_count[i][q] = count;
      }
      Serial.print("("); Serial.print(q); Serial.print(")");//==============================
      Serial.print((curr_count[i][q]) / 430); // curr_count число с плаваюцей точкой можно забирать данные, где i - номер датчика от 0 до 4(5 штук), q - номер канала либо 0, либо 1.
      Serial.print("  ");
    }
    Serial.println(); // вот это не там было
  }
  Serial.println("*** workCount ***");
  Serial.println();
}

#endif // FEATURE_TEMP
